@@ -64,36 +64,6 @@
 #include <stdlib.h>
 
 
-static gchar **html_entities = NULL;
-
-typedef struct
-{
-	gboolean	tags_loaded;
-	const gchar	*tag_file;
-} TagFileInfo;
-
-/* Check before adding any more tags files, usually they should be downloaded separately. */
-enum	/* Geany tag files */
-{
-	GTF_C,
-	GTF_PASCAL,
-	GTF_PHP,
-	GTF_HTML_ENTITIES,
-	GTF_LATEX,
-	GTF_PYTHON,
-	GTF_MAX
-};
-
-static TagFileInfo tag_file_info[GTF_MAX] =
-{
-	{FALSE, "c99.tags"},
-	{FALSE, "pascal.tags"},
-	{FALSE, "php.tags"},
-	{FALSE, "html_entities.tags"},
-	{FALSE, "latex.tags"},
-	{FALSE, "python.tags"}
-};
-
 static GPtrArray *top_level_iter_names = NULL;
 
 enum
@@ -138,7 +108,6 @@ static struct
 }
 symbol_menu;
 
-static void html_tags_loaded(void);
 static void load_user_tags(GeanyFiletypeID ft_id);
 
 /* get the tags_ignore list, exported by tagmanager's options.c */
@@ -201,9 +170,6 @@ static gboolean symbols_load_global_tags(const gchar *tags_file, GeanyFiletype *
  * This provides autocompletion, calltips, etc. */
 void symbols_global_tags_loaded(guint file_type_idx)
 {
-	TagFileInfo *tfi;
-	gint tag_type;
-
 	/* load ignore list for C/C++ parser */
 	if ((file_type_idx == GEANY_FILETYPES_C || file_type_idx == GEANY_FILETYPES_CPP) &&
 		c_tags_ignore == NULL)
@@ -219,55 +185,14 @@ void symbols_global_tags_loaded(guint file_type_idx)
 
 	load_user_tags(file_type_idx);
 
-	switch (file_type_idx)
-	{
-		case GEANY_FILETYPES_PHP:
-		case GEANY_FILETYPES_HTML:
-			html_tags_loaded();
-	}
 	switch (file_type_idx)
 	{
 		case GEANY_FILETYPES_CPP:
 			symbols_global_tags_loaded(GEANY_FILETYPES_C);	/* load C global tags */
-			/* no C++ tagfile yet */
-			return;
-		case GEANY_FILETYPES_C:		tag_type = GTF_C; break;
-		case GEANY_FILETYPES_PASCAL:tag_type = GTF_PASCAL; break;
-		case GEANY_FILETYPES_PHP:	tag_type = GTF_PHP; break;
-		case GEANY_FILETYPES_LATEX:	tag_type = GTF_LATEX; break;
-		case GEANY_FILETYPES_PYTHON:tag_type = GTF_PYTHON; break;
-		default:
-			return;
-	}
-	tfi = &tag_file_info[tag_type];
-
-	if (! tfi->tags_loaded)
-	{
-		gchar *fname = g_build_filename(app->datadir, tfi->tag_file, NULL);
-
-		symbols_load_global_tags(fname, filetypes[file_type_idx]);
-		tfi->tags_loaded = TRUE;
-		g_free(fname);
-	}
-}
-
-
-/* HTML tagfile is just a list of entities for autocompletion (e.g. '&amp;') */
-static void html_tags_loaded(void)
-{
-	TagFileInfo *tfi;
-
-	if (cl_options.ignore_global_tags)
-		return;
-
-	tfi = &tag_file_info[GTF_HTML_ENTITIES];
-	if (! tfi->tags_loaded)
-	{
-		gchar *file = g_build_filename(app->datadir, tfi->tag_file, NULL);
-
-		html_entities = utils_read_file_in_array(file);
-		tfi->tags_loaded = TRUE;
-		g_free(file);
+			break;
+		case GEANY_FILETYPES_PHP:
+			symbols_global_tags_loaded(GEANY_FILETYPES_HTML);	/* load HTML global tags */
+			break;
 	}
 }
 
@@ -326,15 +251,6 @@ const gchar *symbols_get_context_separator(gint ft_id)
 }
 
 
-const gchar **symbols_get_html_entities(void)
-{
-	if (html_entities == NULL)
-		html_tags_loaded(); /* if not yet created, force creation of the array but shouldn't occur */
-
-	return (const gchar **) html_entities;
-}
-
-
 /* sort by name, then line */
 static gint compare_symbol(const TMTag *tag_a, const TMTag *tag_b)
 {
@@ -385,7 +301,6 @@ static gint compare_symbol_lines(gconstpointer a, gconstpointer b)
 static GList *get_tag_list(GeanyDocument *doc, TMTagType tag_types)
 {
 	GList *tag_names = NULL;
-	TMTag *tag;
 	guint i;
 
 	g_return_val_if_fail(doc, NULL);
@@ -395,7 +310,8 @@ static GList *get_tag_list(GeanyDocument *doc, TMTagType tag_types)
 
 	for (i = 0; i < doc->tm_file->tags_array->len; ++i)
 	{
-		tag = TM_TAG(doc->tm_file->tags_array->pdata[i]);
+		TMTag *tag = TM_TAG(doc->tm_file->tags_array->pdata[i]);
+
 		if (G_UNLIKELY(tag == NULL))
 			return NULL;
 
@@ -1762,13 +1678,13 @@ static void init_user_tags(void)
 	const GSList *node;
 	gchar *dir;
 
-	dir = g_build_filename(app->configdir, "tags", NULL);
+	dir = g_build_filename(app->configdir, GEANY_TAGS_SUBDIR, NULL);
 	/* create the user tags dir for next time if it doesn't exist */
 	if (! g_file_test(dir, G_FILE_TEST_IS_DIR))
 		utils_mkdir(dir, FALSE);
 	file_list = utils_get_file_list_full(dir, TRUE, FALSE, NULL);
 
-	SETPTR(dir, g_build_filename(app->datadir, "tags", NULL));
+	SETPTR(dir, g_build_filename(app->datadir, GEANY_TAGS_SUBDIR, NULL));
 	list = utils_get_file_list_full(dir, TRUE, FALSE, NULL);
 	g_free(dir);
 
@@ -2071,102 +1987,113 @@ static TMTag *find_best_goto_tag(GeanyDocument *doc, GPtrArray *tags)
 }
 
 
+static GPtrArray *filter_tags(GPtrArray *tags, TMTag *current_tag, gboolean definition)
+{
+	const TMTagType forward_types = tm_tag_prototype_t | tm_tag_externvar_t;
+	TMTag *tmtag, *last_tag = NULL;
+	GPtrArray *filtered_tags = g_ptr_array_new();
+	guint i;
+
+	foreach_ptr_array(tmtag, i, tags)
+	{
+		if ((definition && !(tmtag->type & forward_types)) ||
+			(!definition && (tmtag->type & forward_types)))
+		{
+			/* If there are typedefs of e.g. a struct such as
+			 * "typedef struct Foo {} Foo;", filter out the typedef unless
+			 * cursor is at the struct name. */
+			if (last_tag != NULL && last_tag->file == tmtag->file &&
+				last_tag->type != tm_tag_typedef_t && tmtag->type == tm_tag_typedef_t)
+			{
+				if (last_tag == current_tag)
+					g_ptr_array_add(filtered_tags, tmtag);
+			}
+			else if (tmtag != current_tag)
+				g_ptr_array_add(filtered_tags, tmtag);
+
+			last_tag = tmtag;
+		}
+	}
+
+	return filtered_tags;
+}
+
+
 static gboolean goto_tag(const gchar *name, gboolean definition)
 {
 	const TMTagType forward_types = tm_tag_prototype_t | tm_tag_externvar_t;
-	TMTagType type;
-	TMTag *tmtag, *last_tag;
+	TMTag *tmtag, *current_tag = NULL;
 	GeanyDocument *old_doc = document_get_current();
 	gboolean found = FALSE;
 	const GPtrArray *all_tags;
-	GPtrArray *workspace_tags, *filtered_tags;
+	GPtrArray *tags, *filtered_tags;
 	guint i;
 	guint current_line = sci_get_current_line(old_doc->editor->sci) + 1;
 
-	/* goto tag definition: all except prototypes / forward declarations / externs */
-	type = (definition) ? tm_tag_max_t - forward_types : forward_types;
-	all_tags = tm_workspace_find(name, NULL, type, NULL, old_doc->file_type->lang);
+	all_tags = tm_workspace_find(name, NULL, tm_tag_max_t, NULL, old_doc->file_type->lang);
 
-	/* get rid of global tags */
-	workspace_tags = g_ptr_array_new();
+	/* get rid of global tags and find tag at current line */
+	tags = g_ptr_array_new();
 	foreach_ptr_array(tmtag, i, all_tags)
 	{
 		if (tmtag->file)
-			g_ptr_array_add(workspace_tags, tmtag);
-	}
-
-	/* If there are typedefs of e.g. a struct such as "typedef struct Foo {} Foo;",
-	 * keep just one of the names in the list - when the cursor is on the struct
-	 * name, keep the typename, otherwise keep the struct name. */
-	last_tag = NULL;
-	filtered_tags = g_ptr_array_new();
-	foreach_ptr_array(tmtag, i, workspace_tags)
-	{
-		if (last_tag != NULL && last_tag->file == tmtag->file &&
-			last_tag->type != tm_tag_typedef_t && tmtag->type == tm_tag_typedef_t)
 		{
-			if (last_tag->line == current_line && filtered_tags->len > 0)
-				/* if cursor on struct, replace struct with the typedef */
-				filtered_tags->pdata[filtered_tags->len-1] = tmtag;
-			/* if cursor anywhere else, use struct (already added) and discard typedef */
+			g_ptr_array_add(tags, tmtag);
+			if (tmtag->file == old_doc->tm_file && tmtag->line == current_line)
+				current_tag = tmtag;
 		}
-		else
-			g_ptr_array_add(filtered_tags, tmtag);
-
-		last_tag = tmtag;
 	}
-	g_ptr_array_free(workspace_tags, TRUE);
-	workspace_tags = filtered_tags;
 
-	if (workspace_tags->len == 1)
+	if (current_tag)
+		/* swap definition/declaration search */
+		definition = current_tag->type & forward_types;
+
+	filtered_tags = filter_tags(tags, current_tag, definition);
+	if (filtered_tags->len == 0)
+	{
+		/* if we didn't find anything, try again with the opposite type */
+		g_ptr_array_free(filtered_tags, TRUE);
+		filtered_tags = filter_tags(tags, current_tag, !definition);
+	}
+	g_ptr_array_free(tags, TRUE);
+	tags = filtered_tags;
+
+	if (tags->len == 1)
 	{
 		GeanyDocument *new_doc;
 
-		tmtag = workspace_tags->pdata[0];
-		new_doc = document_find_by_real_path(
-			tmtag->file->file_name);
+		tmtag = tags->pdata[0];
+		new_doc = document_find_by_real_path(tmtag->file->file_name);
 
-		if (new_doc)
-		{
-			/* If we are already on the tag line, swap definition/declaration */
-			if (new_doc == old_doc && tmtag->line == current_line)
-			{
-				if (goto_tag(name, !definition))
-					found = TRUE;
-			}
-		}
-		else
-		{
+		if (!new_doc)
 			/* not found in opened document, should open */
 			new_doc = document_open_file(tmtag->file->file_name, FALSE, NULL, NULL);
-		}
 
-		if (!found && navqueue_goto_line(old_doc, new_doc, tmtag->line))
-			found = TRUE;
+		navqueue_goto_line(old_doc, new_doc, tmtag->line);
 	}
-	else if (workspace_tags->len > 1)
+	else if (tags->len > 1)
 	{
-		GPtrArray *tags;
+		GPtrArray *tag_list;
 		TMTag *tag, *best_tag;
 
-		g_ptr_array_sort(workspace_tags, compare_tags_by_name_line);
-		best_tag = find_best_goto_tag(old_doc, workspace_tags);
+		g_ptr_array_sort(tags, compare_tags_by_name_line);
+		best_tag = find_best_goto_tag(old_doc, tags);
 
-		tags = g_ptr_array_new();
+		tag_list = g_ptr_array_new();
 		if (best_tag)
-			g_ptr_array_add(tags, best_tag);
-		foreach_ptr_array(tag, i, workspace_tags)
+			g_ptr_array_add(tag_list, best_tag);
+		foreach_ptr_array(tag, i, tags)
 		{
 			if (tag != best_tag)
-				g_ptr_array_add(tags, tag);
+				g_ptr_array_add(tag_list, tag);
 		}
-		show_goto_popup(old_doc, tags, best_tag != NULL);
+		show_goto_popup(old_doc, tag_list, best_tag != NULL);
 
-		g_ptr_array_free(tags, TRUE);
-		found = TRUE;
+		g_ptr_array_free(tag_list, TRUE);
 	}
 
-	g_ptr_array_free(workspace_tags, TRUE);
+	found = tags->len > 0;
+	g_ptr_array_free(tags, TRUE);
 
 	return found;
 }
@@ -2634,7 +2561,6 @@ void symbols_finalize(void)
 {
 	guint i;
 
-	g_strfreev(html_entities);
 	g_strfreev(c_tags_ignore);
 
 	for (i = 0; i < G_N_ELEMENTS(symbols_icons); i++)
