@@ -393,19 +393,13 @@ extern void makeTagEntry (const tagEntryInfo *const tag)
 	int length = 0;
 
 	if (NULL != TagEntryFunction)
-		length = TagEntryFunction(tag);
+		length = TagEntryFunction(tag, TagEntryUserData);
 
 	++TagFile.numTags.added;
 	rememberMaxLengths (strlen (tag->name), (size_t) length);
     }
 }
 
-extern void setTagArglistByName (const char *tag_name, const char *arglist)
-{
-    if (NULL != TagEntrySetArglistFunction)
-	TagEntrySetArglistFunction(tag_name, arglist);
-}
-
 extern void initTagEntry (tagEntryInfo *const e, const char *const name)
 {
     Assert (File.source.name != NULL);
