@@ -31,14 +31,21 @@
 */
 static parserDefinitionFunc* BuiltInParsers[] = { PARSER_LIST };
 parserDefinition** LanguageTable = NULL;
-static unsigned int LanguageCount = 0;
+unsigned int LanguageCount = 0;
 tagEntryFunction TagEntryFunction = NULL;
-tagEntrySetArglistFunction TagEntrySetArglistFunction = NULL;
+void *TagEntryUserData = NULL;
 
 /*
 *   FUNCTION DEFINITIONS
 */
 
+extern void setTagEntryFunction(tagEntryFunction entry_function, void *user_data)
+{
+    TagEntryFunction = entry_function;
+    TagEntryUserData = user_data;
+}
+
+
 extern void makeSimpleTag (const vString* const name,
 			   kindOption* const kinds, const int kind)
 {
